/***************************************************************************
*   Copyright (C) 2004 by Denis A. Birukov aka Messenger of death         *
*   riper@fromru.com                                                      *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef ME_N_STORAGEME_C_STORAGE_H
#define ME_N_STORAGEME_C_STORAGE_H
#include <me_defines.h>
#include <me_c_storage_driver_interface.h>
#include <string>
#include <map>
#include <iostream>
#include <me_c_storageentry.h>
#include <me_debug.h>
namespace ME_n_Storage
{
	class ME_c_Parser;
	class ME_c_Storage_Driver_Interface;
	class ME_c_Storage_Entry;
	enum ME_STORAGE_ERRORS
	{
	    ME_STORAGE_TYPE_MISMATCH = 1,
	    ME_STORAGE_NO_ENTRY
	};
	enum ME_STORAGE_TYPE
	{
	    ME_STORAGE_NONE = 0,
	    ME_STORAGE_UINT32,
	    ME_STORAGE_SINT32,
	    ME_STORAGE_UINT16,
	    ME_STORAGE_SINT16,
	    ME_STORAGE_UINT8,
	    ME_STORAGE_SINT8,
	    ME_STORAGE_STRING,
	    ME_STORAGE_DOUBLE
	};
	/**
	      .
	  -   .
	    .
	       -
	 
	@author Denis A. Birukov aka Messenger of death 
	*/
	class ME_c_Storage
	{
		public:
			/**   ,   ,   
			*/
			DECLSPEC ME_c_Storage( ME_c_Storage_Driver_Interface* drv );
			/**   ,   ,    .   
			extern "C" DECLSPEC ME_c_Storage_Driver_Interface* _create_storage_driver()
			*/
			DECLSPEC ME_c_Storage( std::string str );
			DECLSPEC ~ME_c_Storage();
			/**         .
			    - *,          
			(   Drivers->OpenGL       *,        Drivers->OpenGL->`valuename`.
			     ,       .*/
			DECLSPEC bool _register_parser( const std::string category, ME_c_Parser* parser );
			/**         .
			    - *,          
			(   Drivers->OpenGL       *,        Drivers->OpenGL->`valuename`.
			     ,       .    .
			*/
			DECLSPEC bool _register_parser( const std::string category );
			/**     .*/
			DECLSPEC bool _unregister_parser( const std::string category );
			/**    ,      .    .*/
			DECLSPEC bool _add_value( const std::string name, ME_c_Storage_Entry* );
			/**    .      .*/
			DECLSPEC bool _add_value( const std::string name, const ME_c_Storage_Entry& );
			/**     .*/
			DECLSPEC bool _modify_value( const std::string name, const ME_c_Storage_Entry& );
			/**    */
			DECLSPEC bool _remove_entry ( const std::string name );
			/** ,   */
			DECLSPEC ME_c_Storage_Entry* operator[] ( const std::string name );
			/**  true    */
			DECLSPEC bool exist( const std::string name );
			/**     .     ,         , 
			   ME_Exception    
			ME_STORAGE_TYPE_MISMATCH  ME_STORAGE_NO_ENTRY.
			     def     entry_type,    ME_STORAGE_NONE. 
			*/
			DECLSPEC Uint32 _get_value( const std::string name, const Uint32 &def, bool rise_exception = false );
			DECLSPEC Sint32 _get_value( const std::string name, const Sint32 &def, bool rise_exception = false );
			DECLSPEC double _get_value( const std::string name, const double &def, bool rise_exception = false );
			DECLSPEC std::string _get_value( const std::string name, const std::string &def, bool rise_exception = false );

			/**  */
			DECLSPEC const std::map<std::string, ME_c_Storage_Entry*>& _get_all();
			/**     */
			DECLSPEC bool operator() ( std::istream &stream );
			/**    */
			DECLSPEC const ME_c_Storage_Driver_Interface& _get_driver();
			/** */
			DECLSPEC bool _set_driver( ME_c_Storage_Driver_Interface *drv );
			/**   ,           exist.      std::string.*/
			DECLSPEC ME_c_Storage_Entry& operator*();


		private:
			std::map<std::string, ME_c_Storage_Entry*> _values_;
			std::map<std::string, ME_c_Parser*> _parsers_;
			ME_c_Storage_Driver_Interface *_storage_driver_;
			std::string _latest_;
			template <class T> T _get_value_( const std::string name, const T &def, ME_STORAGE_TYPE entry_type = ME_STORAGE_NONE, bool rise_exception = false );
	};
}

#endif
